@@ -26,8 +26,8 @@
 
 /* forward declarations */
 static double read_ieee_extended(sox_format_t *);
-static int aiffwriteheader(sox_format_t *, size_t);
-static int aifcwriteheader(sox_format_t *, size_t);
+static int aiffwriteheader(sox_format_t *, uint64_t);
+static int aifcwriteheader(sox_format_t *, uint64_t);
 static void write_ieee_extended(sox_format_t *, double);
 static double ConvertFromIeeeExtended(unsigned char*);
 static void ConvertToIeeeExtended(double, char *);
@@ -41,6 +41,7 @@ int lsx_aiffstartread(sox_format_t * ft)
   uint32_t totalsize;
   uint32_t chunksize;
   unsigned short channels = 0;
+  sox_encoding_t enc = SOX_ENCODING_SIGN2;
   uint32_t frames;
   unsigned short bits = 0;
   double rate = 0.0;
@@ -111,10 +112,28 @@ int lsx_aiffstartread(sox_format_t * ft)
           /* Need to endian swap all the data */
           is_sowt = 1;
         }
+        else if (strncmp(buf, "fl32", (size_t)4) == 0 ||
+            strncmp(buf, "FL32", (size_t)4) == 0) {
+          enc = SOX_ENCODING_FLOAT;
+          if (bits != 32) {
+            lsx_fail_errno(ft, SOX_EHDR,
+              "Sample size of %u is not consistent with `fl32' compression type", bits);
+            return SOX_EOF;
+          }
+        }
+        else if (strncmp(buf, "fl64", (size_t)4) == 0 ||
+            strncmp(buf, "FL64", (size_t)4) == 0) {
+          enc = SOX_ENCODING_FLOAT;
+          if (bits != 64) {
+            lsx_fail_errno(ft, SOX_EHDR,
+              "Sample size of %u is not consistent with `fl64' compression type", bits);
+            return SOX_EOF;
+          }
+        }
         else if (strncmp(buf, "NONE", (size_t)4) != 0 &&
             strncmp(buf, "twos", (size_t)4) != 0) {
           buf[4] = 0;
-          lsx_fail_errno(ft,SOX_EHDR,"AIFC files that contain compressed data are not supported: %s",buf);
+          lsx_fail_errno(ft, SOX_EHDR, "Unsupported AIFC compression type `%s'", buf);
           return(SOX_EOF);
         }
       }
@@ -331,13 +350,11 @@ int lsx_aiffstartread(sox_format_t * ft)
   }
 
   if (foundcomm) {
-    if (ft->encoding.encoding != SOX_ENCODING_UNKNOWN && ft->encoding.encoding != SOX_ENCODING_SIGN2)
-      lsx_report("AIFF only supports signed data.  Forcing to signed.");
-    ft->encoding.encoding = SOX_ENCODING_SIGN2;
     if      (bits <=  8) bits = 8;
     else if (bits <= 16) bits = 16;
     else if (bits <= 24) bits = 24;
     else if (bits <= 32) bits = 32;
+    else if (bits == 64 && enc == SOX_ENCODING_FLOAT) /* no-op */;
     else {
       lsx_fail_errno(ft,SOX_EFMT,"unsupported sample size in AIFF header: %d", bits);
       return(SOX_EOF);
@@ -369,20 +386,20 @@ int lsx_aiffstartread(sox_format_t * ft)
     foundinstr = 0;
   }
   if (foundmark && foundinstr) {
-    int i;
+    int i2;
     int slbIndex = 0, sleIndex = 0;
     int rlbIndex = 0, rleIndex = 0;
 
     /* find our loop markers and save their marker indexes */
-    for(i = 0; i < nmarks; i++) {
-      if(marks[i].id == sustainLoopBegin)
-        slbIndex = i;
-      if(marks[i].id == sustainLoopEnd)
-        sleIndex = i;
-      if(marks[i].id == releaseLoopBegin)
-        rlbIndex = i;
-      if(marks[i].id == releaseLoopEnd)
-        rleIndex = i;
+    for(i2 = 0; i2 < nmarks; i2++) {
+      if(marks[i2].id == sustainLoopBegin)
+        slbIndex = i2;
+      if(marks[i2].id == sustainLoopEnd)
+        sleIndex = i2;
+      if(marks[i2].id == releaseLoopBegin)
+        rlbIndex = i2;
+      if(marks[i2].id == releaseLoopEnd)
+        rleIndex = i2;
     }
 
     ft->oob.instr.nloops = 0;
@@ -408,7 +425,7 @@ int lsx_aiffstartread(sox_format_t * ft)
   reportInstrument(ft);
 
   return lsx_check_read_params(
-      ft, channels, rate, SOX_ENCODING_SIGN2, bits, (off_t)ssndsize, sox_false);
+      ft, channels, rate, enc, bits, (uint64_t)ssndsize, sox_false);
 }
 
 /* print out the MIDI key allocations, loop points, directions etc */
@@ -582,7 +599,7 @@ int lsx_aiffstartwrite(sox_format_t * ft)
            At 48 kHz, 16 bits stereo, this gives ~3 hours of audio.
            Sorry, the AIFF format does not provide for an indefinite
            number of samples. */
-        return(aiffwriteheader(ft, (size_t) 0x7f000000 / ((ft->encoding.bits_per_sample>>3)*ft->signal.channels)));
+        return(aiffwriteheader(ft, (uint64_t) 0x7f000000 / ((ft->encoding.bits_per_sample>>3)*ft->signal.channels)));
 }
 
 int lsx_aiffstopwrite(sox_format_t * ft)
@@ -608,13 +625,14 @@ int lsx_aiffstopwrite(sox_format_t * ft)
         return(aiffwriteheader(ft, ft->olength / ft->signal.channels));
 }
 
-static int aiffwriteheader(sox_format_t * ft, size_t nframes)
+static int aiffwriteheader(sox_format_t * ft, uint64_t nframes)
 {
         int hsize =
                 8 /*COMM hdr*/ + 18 /*COMM chunk*/ +
                 8 /*SSND hdr*/ + 12 /*SSND chunk*/;
         unsigned bits = 0;
         unsigned i;
+        uint64_t size;
         size_t padded_comment_size = 0, comment_size = 0;
         size_t comment_chunk_size = 0;
         char * comment = lsx_cat_comments(ft->oob.comments);
@@ -660,7 +678,13 @@ static int aiffwriteheader(sox_format_t * ft, size_t nframes)
 
         lsx_writes(ft, "FORM"); /* IFF header */
         /* file size */
-        lsx_writedw(ft, (unsigned) (hsize + nframes * (ft->encoding.bits_per_sample >> 3) * ft->signal.channels));
+        size = hsize + nframes * (ft->encoding.bits_per_sample >> 3) * ft->signal.channels;
+        if (size > UINT_MAX)
+        {
+            lsx_warn("file size too big for accurate AIFF header");
+            size = UINT_MAX;
+        }
+        lsx_writedw(ft, (unsigned)size);
         lsx_writes(ft, "AIFF"); /* File type */
 
         /* Now we write the COMT comment chunk using the precomputed sizes */
@@ -705,12 +729,18 @@ static int aiffwriteheader(sox_format_t * ft, size_t nframes)
                 lsx_writew(ft, ft->oob.instr.nloops);
 
                 for(i = 0; i < ft->oob.instr.nloops; i++) {
+                        unsigned start = ft->oob.loops[i].start > UINT_MAX
+                            ? UINT_MAX
+                            : ft->oob.loops[i].start;
+                        unsigned end = ft->oob.loops[i].start + ft->oob.loops[i].length > UINT_MAX
+                            ? UINT_MAX
+                            : ft->oob.loops[i].start + ft->oob.loops[i].length;
                         lsx_writew(ft, i + 1);
-                        lsx_writedw(ft, (unsigned) ft->oob.loops[i].start);
+                        lsx_writedw(ft, start);
                         lsx_writeb(ft, 0);
                         lsx_writeb(ft, 0);
                         lsx_writew(ft, i*2 + 1);
-                        lsx_writedw(ft, (unsigned) (ft->oob.loops[i].start + ft->oob.loops[i].length));
+                        lsx_writedw(ft, end);
                         lsx_writeb(ft, 0);
                         lsx_writeb(ft, 0);
                 }
@@ -766,7 +796,7 @@ int lsx_aifcstartwrite(sox_format_t * ft)
            At 48 kHz, 16 bits stereo, this gives ~3 hours of music.
            Sorry, the AIFC format does not provide for an "infinite"
            number of samples. */
-        return(aifcwriteheader(ft, (size_t) 0x7f000000 / ((ft->encoding.bits_per_sample >> 3)*ft->signal.channels)));
+        return(aifcwriteheader(ft, (uint64_t) 0x7f000000 / ((ft->encoding.bits_per_sample >> 3)*ft->signal.channels)));
 }
 
 int lsx_aifcstopwrite(sox_format_t * ft)
@@ -792,12 +822,13 @@ int lsx_aifcstopwrite(sox_format_t * ft)
         return(aifcwriteheader(ft, ft->olength / ft->signal.channels));
 }
 
-static int aifcwriteheader(sox_format_t * ft, size_t nframes)
+static int aifcwriteheader(sox_format_t * ft, uint64_t nframes)
 {
-        unsigned hsize =
-                12 /*FVER*/ + 8 /*COMM hdr*/ + 18+4+1+15 /*COMM chunk*/ +
-                8 /*SSND hdr*/ + 12 /*SSND chunk*/;
+        unsigned hsize;
         unsigned bits = 0;
+        uint64_t size;
+        char *ctype = NULL, *cname = NULL;
+        unsigned cname_len = 0, comm_len = 0, comm_padding = 0;
 
         if (ft->encoding.encoding == SOX_ENCODING_SIGN2 &&
             ft->encoding.bits_per_sample == 8)
@@ -811,15 +842,52 @@ static int aifcwriteheader(sox_format_t * ft, size_t nframes)
         else if (ft->encoding.encoding == SOX_ENCODING_SIGN2 &&
                  ft->encoding.bits_per_sample == 32)
                 bits = 32;
+        else if (ft->encoding.encoding == SOX_ENCODING_FLOAT &&
+                 ft->encoding.bits_per_sample == 32)
+                bits = 32;
+        else if (ft->encoding.encoding == SOX_ENCODING_FLOAT &&
+                 ft->encoding.bits_per_sample == 64)
+                bits = 64;
         else
         {
                 lsx_fail_errno(ft,SOX_EFMT,"unsupported output encoding/size for AIFC header");
                 return(SOX_EOF);
         }
 
+        /* calculate length of COMM chunk (without header) */
+        switch (ft->encoding.encoding) {
+          case SOX_ENCODING_SIGN2:
+            ctype = "NONE";
+            cname = "not compressed";
+            break;
+          case SOX_ENCODING_FLOAT:
+            if (bits == 32) {
+              ctype = "fl32";
+              cname = "32-bit floating point";
+            } else {
+              ctype = "fl64";
+              cname = "64-bit floating point";
+            }
+            break;
+          default: /* can't happen */
+            break;
+        }
+        cname_len = strlen(cname);
+        comm_len = 18+4+1+cname_len;
+        comm_padding = comm_len%2;
+
+        hsize = 12 /*FVER*/ + 8 /*COMM hdr*/ + comm_len+comm_padding /*COMM chunk*/ +
+                8 /*SSND hdr*/ + 12 /*SSND chunk*/;
+
         lsx_writes(ft, "FORM"); /* IFF header */
         /* file size */
-        lsx_writedw(ft, (unsigned) (hsize + nframes * (ft->encoding.bits_per_sample >> 3) * ft->signal.channels));
+        size = hsize + nframes * (ft->encoding.bits_per_sample >> 3) * ft->signal.channels;
+        if (size > UINT_MAX)
+        {
+            lsx_warn("file size too big for accurate AIFC header");
+            size = UINT_MAX;
+        }
+        lsx_writedw(ft, (unsigned)size);
         lsx_writes(ft, "AIFC"); /* File type */
 
         /* FVER chunk */
@@ -829,16 +897,17 @@ static int aifcwriteheader(sox_format_t * ft, size_t nframes)
 
         /* COMM chunk -- describes encoding (and #frames) */
         lsx_writes(ft, "COMM");
-        lsx_writedw(ft, 18+4+1+15); /* COMM chunk size */
+        lsx_writedw(ft, comm_len+comm_padding); /* COMM chunk size */
         lsx_writew(ft, ft->signal.channels); /* nchannels */
         lsx_writedw(ft, (unsigned) nframes); /* number of frames */
         lsx_writew(ft, bits); /* sample width, in bits */
         write_ieee_extended(ft, (double)ft->signal.rate);
 
-        lsx_writes(ft, "NONE"); /*compression_type*/
-        lsx_writeb(ft, 14);
-        lsx_writes(ft, "not compressed");
-        lsx_writeb(ft, 0);
+        lsx_writes(ft, ctype); /*compression_type*/
+        lsx_writeb(ft, cname_len);
+        lsx_writes(ft, cname);
+        if (comm_padding)
+          lsx_writeb(ft, 0);
 
         /* SSND chunk -- describes data */
         lsx_writes(ft, "SSND");
@@ -879,8 +948,17 @@ static void write_ieee_extended(sox_format_t * ft, double x)
  */
 
 /* Copyright (C) 1988-1991 Apple Computer, Inc.
+ *
  * All rights reserved.
  *
+ * Warranty Information
+ *  Even though Apple has reviewed this software, Apple makes no warranty
+ *  or representation, either express or implied, with respect to this
+ *  software, its quality, accuracy, merchantability, or fitness for a
+ *  particular purpose.  As a result, this software is provided "as is,"
+ *  and you, its user, are assuming the entire risk as to its quality
+ *  and accuracy.
+ *
  * Machine-independent I/O routines for IEEE floating-point numbers.
  *
  * NaN's and infinities are converted to HUGE_VAL, which
@@ -968,8 +1046,20 @@ static void ConvertToIeeeExtended(double num, char *bytes)
 
 /*
  * Copyright (C) 1988-1991 Apple Computer, Inc.
+ *
  * All rights reserved.
  *
+ * Warranty Information
+ *  Even though Apple has reviewed this software, Apple makes no warranty
+ *  or representation, either express or implied, with respect to this
+ *  software, its quality, accuracy, merchantability, or fitness for a
+ *  particular purpose.  As a result, this software is provided "as is,"
+ *  and you, its user, are assuming the entire risk as to its quality
+ *  and accuracy.
+ *
+ * This code may be used and freely distributed as long as it includes
+ * this copyright notice and the above warranty information.
+ *
  * Machine-independent I/O routines for IEEE floating-point numbers.
  *
  * NaN's and infinities are converted to HUGE_VAL, which
