@@ -24,7 +24,7 @@
 
 /* Private data for fade file */
 typedef struct { /* These are measured as samples */
-    size_t in_start, in_stop, out_start, out_stop, samplesdone;
+    uint64_t in_start, in_stop, out_start, out_stop, samplesdone;
     char *in_stop_str, *out_start_str, *out_stop_str;
     char in_fadetype, out_fadetype;
     char do_out;
@@ -32,7 +32,7 @@ typedef struct { /* These are measured as samples */
 } priv_t;
 
 /* prototypes */
-static double fade_gain(size_t index, size_t range, int fadetype);
+static double fade_gain(uint64_t index, uint64_t range, int fadetype);
 
 /*
  * Process options
@@ -47,6 +47,8 @@ static int sox_fade_getopts(sox_effect_t * effp, int argc, char **argv)
     priv_t * fade = (priv_t *) effp->priv;
     char t_char[2];
     int t_argno;
+    uint64_t samples;
+    const char *n;
   --argc, ++argv;
 
     if (argc < 1 || argc > 4)
@@ -71,12 +73,13 @@ static int sox_fade_getopts(sox_effect_t * effp, int argc, char **argv)
         fade->out_fadetype = 'l';
     }
 
-    fade->in_stop_str = lsx_malloc(strlen(argv[0])+1);
-    strcpy(fade->in_stop_str,argv[0]);
+    fade->in_stop_str = lsx_strdup(argv[0]);
     /* Do a dummy parse to see if it will fail */
-    if (lsx_parsesamples(0., fade->in_stop_str, &fade->in_stop, 't') == NULL)
+    n = lsx_parsesamples(0., fade->in_stop_str, &samples, 't');
+    if (!n || *n)
       return lsx_usage(effp);
 
+    fade->in_stop = samples;
     fade->out_start_str = fade->out_stop_str = 0;
 
     for (t_argno = 1; t_argno < argc && t_argno < 3; t_argno++)
@@ -84,23 +87,23 @@ static int sox_fade_getopts(sox_effect_t * effp, int argc, char **argv)
         /* See if there is fade-in/fade-out times/curves specified. */
         if(t_argno == 1)
         {
-            fade->out_stop_str = lsx_malloc(strlen(argv[t_argno])+1);
-            strcpy(fade->out_stop_str,argv[t_argno]);
+            fade->out_stop_str = lsx_strdup(argv[t_argno]);
 
             /* Do a dummy parse to see if it will fail */
-            if (lsx_parsesamples(0., fade->out_stop_str,
-                                &fade->out_stop, 't') == NULL)
+            n = lsx_parsesamples(0., fade->out_stop_str, &samples, 't');
+            if (!n || *n)
               return lsx_usage(effp);
+            fade->out_stop = samples;
         }
         else
         {
-            fade->out_start_str = lsx_malloc(strlen(argv[t_argno])+1);
-            strcpy(fade->out_start_str,argv[t_argno]);
+            fade->out_start_str = lsx_strdup(argv[t_argno]);
 
             /* Do a dummy parse to see if it will fail */
-            if (lsx_parsesamples(0., fade->out_start_str,
-                                &fade->out_start, 't') == NULL)
+            n = lsx_parsesamples(0., fade->out_start_str, &samples, 't');
+            if (!n || *n)
               return lsx_usage(effp);
+            fade->out_start = samples;
         }
     } /* End for(t_argno) */
 
@@ -115,24 +118,29 @@ static int sox_fade_start(sox_effect_t * effp)
 {
     priv_t * fade = (priv_t *) effp->priv;
     sox_bool truncate = sox_false;
+    uint64_t samples;
 
     /* converting time values to samples */
     fade->in_start = 0;
     if (lsx_parsesamples(effp->in_signal.rate, fade->in_stop_str,
-                        &fade->in_stop, 't') == NULL)
+                        &samples, 't') == NULL)
       return lsx_usage(effp);
 
+    fade->in_stop = samples;
     fade->do_out = 0;
     /* See if user specified a stop time */
     if (fade->out_stop_str)
     {
         fade->do_out = 1;
         if (lsx_parsesamples(effp->in_signal.rate, fade->out_stop_str,
-                            &fade->out_stop, 't') == NULL)
+                            &samples, 't') == NULL)
           return lsx_usage(effp);
+        fade->out_stop = samples;
 
         if (!(truncate = !!fade->out_stop)) {
-          fade->out_stop = effp->in_signal.length / effp->in_signal.channels;
+          fade->out_stop = effp->in_signal.length != SOX_UNKNOWN_LEN ?
+              effp->in_signal.length / effp->in_signal.channels :
+              0;
           if (!fade->out_stop) {
             lsx_fail("cannot fade out: audio length is neither known nor given");
             return SOX_EOF;
@@ -143,11 +151,10 @@ static int sox_fade_start(sox_effect_t * effp)
         if (fade->out_start_str)
         {
             if (lsx_parsesamples(effp->in_signal.rate, fade->out_start_str,
-                        &fade->out_start, 't') == NULL)
+                        &samples, 't') == NULL)
               return lsx_usage(effp);
             /* Fade time is relative to stop time. */
-            fade->out_start = fade->out_stop - fade->out_start;
-
+            fade->out_start = fade->out_stop - samples;
         }
         else
             /* If user doesn't specify fade out length then
@@ -162,22 +169,29 @@ static int sox_fade_start(sox_effect_t * effp)
          */
         fade->out_stop = 0;
 
-    /* Sanity check for fade times vs total time */
-    if (fade->in_stop > fade->out_start && fade->out_start != 0)
-    { /* Fades too long */
-        lsx_fail("Fade: End of fade-in should not happen before beginning of fade-out");
-        return(SOX_EOF);
-    } /* endif fade time sanity */
+    if (fade->out_start) {              /* Sanity check */
+      if (fade->in_stop > fade->out_start)
+        --fade->in_stop;                /* 1 sample grace for rounding error. */
+      if (fade->in_stop > fade->out_start) {
+        lsx_fail("fade-out overlaps fade-in");
+        return SOX_EOF;
+      }
+    }
 
     fade->samplesdone = fade->in_start;
     fade->endpadwarned = 0;
 
-    lsx_debug("fade: in_start = %lu in_stop = %lu out_start = %lu out_stop = %lu", (unsigned long)fade->in_start, (unsigned long)fade->in_stop, (unsigned long)fade->out_start, (unsigned long)fade->out_stop);
+    lsx_debug("in_start = %" PRIu64 " in_stop = %" PRIu64 " "
+      "out_start = %" PRIu64 " out_stop = %" PRIu64,
+      fade->in_start, fade->in_stop, fade->out_start, fade->out_stop);
 
     if (fade->in_start == fade->in_stop && !truncate &&
         fade->out_start == fade->out_stop)
       return SOX_EFF_NULL;
 
+    effp->out_signal.length = truncate ?
+        fade->out_stop * effp->in_signal.channels : effp->in_signal.length;
+
     return SOX_SUCCESS;
 }
 
@@ -273,12 +287,13 @@ static int sox_fade_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp
     size_t t_chan = 0;
 
     len = *osamp;
+    len -= len % effp->in_signal.channels;
     *osamp = 0;
 
     if (fade->do_out && fade->samplesdone < fade->out_stop &&
         !(fade->endpadwarned))
     { /* Warning about padding silence into end of sample */
-        lsx_warn("Fade: warning: End time passed end-of-file. Padding with silence");
+        lsx_warn("End time past end of audio. Padding with silence");
         fade->endpadwarned = 1;
     } /* endif endpadwarned */
 
@@ -320,7 +335,7 @@ static int lsx_kill(sox_effect_t * effp)
 /* Function returns gain value 0.0 - 1.0 according index / range ratio
 * and -1.0 if  type is invalid
 * todo: to optimize performance calculate gain every now and then and interpolate */
-static double fade_gain(size_t index, size_t range, int type)
+static double fade_gain(uint64_t index, uint64_t range, int type)
 {
     double retval = 0.0, findex = 0.0;
 
