@@ -21,7 +21,6 @@
 
 #include "sox_i.h"
 #include "fft4g.h"
-#include "sgetopt.h"
 #include <assert.h>
 #include <math.h>
 #ifdef HAVE_LIBPNG_PNG_H
@@ -34,6 +33,8 @@
 #define MAX_FFT_SIZE 4096
 #define is_p2(x) !(x & (x - 1))
 
+#define MAX_X_SIZE 200000
+
 typedef enum {Window_Hann, Window_Hamming, Window_Bartlett, Window_Rectangular, Window_Kaiser} win_type_t;
 static lsx_enum_item const window_options[] = {
   LSX_ENUM_ITEM(Window_,Hann)
@@ -57,7 +58,7 @@ typedef struct {
 
   /* Per-channel work area */
   int        WORK;  /* Start of work area is marked by this dummy variable. */
-  size_t     skip;
+  uint64_t   skip;
   int        dft_size, step_size, block_steps, block_num, rows, cols, read;
   int        x_size, end, end_min, last_end;
   sox_bool   truncated;
@@ -93,24 +94,26 @@ static unsigned char const alt_palette[] =
 static int getopts(sox_effect_t * effp, int argc, char **argv)
 {
   priv_t * p = (priv_t *)effp->priv;
-  size_t duration;
+  uint64_t duration;
   char const * next;
   int c;
+  lsx_getopt_t optstate;
+  lsx_getopt_init(argc, argv, "+S:d:x:X:y:Y:z:Z:q:p:W:w:st:c:AarmlhTo:", NULL, lsx_getopt_flag_none, 1, &optstate);
 
   p->dB_range = 120, p->spectrum_points = 249, p->perm = 1; /* Non-0 defaults */
   p->out_name = "spectrogram.png", p->comment = "Created by SoX";
 
-  while ((c = lsx_getopt(argc, argv, "+S:d:x:X:y:Y:z:Z:q:p:W:w:st:c:AarmlhTo:")) != -1) switch (c) {
-    GETOPT_NUMERIC('x', x_size0       , 100, 5000)
-    GETOPT_NUMERIC('X', pixels_per_sec,  1 , 5000)
-    GETOPT_NUMERIC('y', y_size        , 64 , 1200)
-    GETOPT_NUMERIC('Y', Y_size        , 130, MAX_FFT_SIZE / 2 + 2)
-    GETOPT_NUMERIC('z', dB_range      , 20 , 180)
-    GETOPT_NUMERIC('Z', gain          ,-100, 100)
-    GETOPT_NUMERIC('q', spectrum_points, 0 , p->spectrum_points)
-    GETOPT_NUMERIC('p', perm          ,  1 , 6)
-    GETOPT_NUMERIC('W', window_adjust , -10, 10)
-    case 'w': p->win_type = lsx_enum_option(c, window_options);   break;
+  while ((c = lsx_getopt(&optstate)) != -1) switch (c) {
+    GETOPT_NUMERIC(optstate, 'x', x_size0       , 100, MAX_X_SIZE)
+    GETOPT_NUMERIC(optstate, 'X', pixels_per_sec,  1 , 5000)
+    GETOPT_NUMERIC(optstate, 'y', y_size        , 64 , 1200)
+    GETOPT_NUMERIC(optstate, 'Y', Y_size        , 130, MAX_FFT_SIZE / 2 + 2)
+    GETOPT_NUMERIC(optstate, 'z', dB_range      , 20 , 180)
+    GETOPT_NUMERIC(optstate, 'Z', gain          ,-100, 100)
+    GETOPT_NUMERIC(optstate, 'q', spectrum_points, 0 , p->spectrum_points)
+    GETOPT_NUMERIC(optstate, 'p', perm          ,  1 , 6)
+    GETOPT_NUMERIC(optstate, 'W', window_adjust , -10, 10)
+    case 'w': p->win_type = lsx_enum_option(c, optstate.arg, window_options);   break;
     case 's': p->slack_overlap    = sox_true;   break;
     case 'A': p->alt_palette      = sox_true;   break;
     case 'a': p->no_axes          = sox_true;   break;
@@ -119,16 +122,16 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
     case 'l': p->light_background = sox_true;   break;
     case 'h': p->high_colour      = sox_true;   break;
     case 'T': p->truncate         = sox_true;   break;
-    case 't': p->title            = lsx_optarg; break;
-    case 'c': p->comment          = lsx_optarg; break;
-    case 'o': p->out_name         = lsx_optarg; break;
-    case 'S': next = lsx_parsesamples(1e5, lsx_optarg, &duration, 't');
+    case 't': p->title            = optstate.arg; break;
+    case 'c': p->comment          = optstate.arg; break;
+    case 'o': p->out_name         = optstate.arg; break;
+    case 'S': next = lsx_parsesamples(1e5, optstate.arg, &duration, 't');
       if (next && !*next) {p->start_time = duration * 1e-5; break;}
       return lsx_usage(effp);
-    case 'd': next = lsx_parsesamples(1e5, lsx_optarg, &duration, 't');
+    case 'd': next = lsx_parsesamples(1e5, optstate.arg, &duration, 't');
       if (next && !*next) {p->duration = duration * 1e-5; break;}
       return lsx_usage(effp);
-    default: lsx_fail("invalid option `-%c'", optopt); return lsx_usage(effp);
+    default: lsx_fail("invalid option `-%c'", optstate.opt); return lsx_usage(effp);
   }
   if (!!p->x_size0 + !!p->pixels_per_sec + !!p->duration > 2) {
     lsx_fail("only two of -x, -X, -d may be given");
@@ -144,7 +147,7 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
   if (p->alt_palette)
     p->spectrum_points = min(p->spectrum_points, (int)alt_palette_len);
   p->shared_ptr = &p->shared;
-  return lsx_optind !=argc || p->win_type == INT_MAX? lsx_usage(effp) : SOX_SUCCESS;
+  return optstate.ind !=argc || p->win_type == INT_MAX? lsx_usage(effp) : SOX_SUCCESS;
 }
 
 static double make_window(priv_t * p, int end)
@@ -195,15 +198,15 @@ static int start(sox_effect_t * effp)
   double actual, duration = p->duration, pixels_per_sec = p->pixels_per_sec;
 
   memset(&p->WORK, 0, sizeof(*p) - field_offset(priv_t, WORK));
-  
+
   p->skip = p->start_time * effp->in_signal.rate + .5;
   p->x_size = p->x_size0;
   while (sox_true) {
     if (!pixels_per_sec && p->x_size && duration)
       pixels_per_sec = min(5000, p->x_size / duration);
     else if (!p->x_size && pixels_per_sec && duration)
-      p->x_size = min(5000, (int)(pixels_per_sec * duration + .5));
-    if (!duration && effp->in_signal.length) {
+      p->x_size = min(MAX_X_SIZE, (int)(pixels_per_sec * duration + .5));
+    if (!duration && effp->in_signal.length != SOX_UNKNOWN_LEN) {
       duration = effp->in_signal.length / (effp->in_signal.rate * effp->in_signal.channels);
       duration -= p->start_time;
       if (duration <= 0)
@@ -518,7 +521,7 @@ static int stop(sox_effect_t * effp)
   memset(pixels, Background, cols * rows * sizeof(*pixels));
   png_init_io(png, file);
   png_set_PLTE(png, png_info, palette, fixed_palette + p->spectrum_points);
-  png_set_IHDR(png, png_info, (size_t)cols, (size_t)rows, 8,
+  png_set_IHDR(png, png_info, (png_uint_32)cols, (png_uint_32)rows, 8,
       PNG_COLOR_TYPE_PALETTE, PNG_INTERLACE_NONE,
       PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
   for (j = 0; j < rows; ++j)               /* Put (0,0) at bottom-left of PNG */
